package org.javanicus.blog;
import java.util.Properties;
import java.util.Date;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.io.*;
import java.sql.*;

import com.oroinc.net.ftp.*;

public class Blogger implements BlogCallback {
	private String previousDate = "";
	private boolean firstItem = true;
	private Connection con = null;
    private BlogDestination destination = null;

	private  Connection getConnection() throws Exception{
		if (con == null) {
			Driver driver = (Driver)Class.forName("org.hsqldb.jdbcDriver").newInstance();
			Properties p = new Properties();
			p.put("user","sa");
			p.put("password","");
			String url = "jdbc:hsqldb:blog";
			con = driver.connect(url,p);
			con.setAutoCommit(true);
		}
		return con;
	}

	private  void store(BlogItem item) {
		try {
			Connection con = getConnection();
			PreparedStatement pstmt = con.prepareStatement("insert into item (blog_id,id,when,description) values (?,?,?,?)");
			pstmt.setString(1,destination.getProject());
			pstmt.setLong(2,item.getKey());
			pstmt.setTimestamp(3,new Timestamp(item.getDateValue().getTime()));
			pstmt.setString(4,item.getDescription());
			pstmt.executeUpdate();
			pstmt.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private  List readItems() {
		List rv = new ArrayList();
		try {
			Connection con = getConnection();
			Statement stmt = con.createStatement();
			ResultSet rs = stmt.executeQuery("select id,when,description from item where blog_id = 'javanicus' order by when desc");
			while (rs.next()) {
				long id = rs.getLong(1);
				Timestamp when = rs.getTimestamp(2);
				Date date = new Date(when.getTime());
				String description = rs.getString(3);

				rv.add(new BlogItem(id,date,description));
			}
			stmt.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return rv;
	}

    private void printXMLHeader(PrintStream out) {
        out.println("<?xml version=\"1.0\"?>");
        out.println("<!-- RSS generated by localBlog on " + new Date() + " -->");
        out.println("<rss version=\"0.92\">");
        out.println("<channel>");
		out.println("<title>" + destination.getProject() + "</title>");
		out.println("<link>" + destination.getWebserver() + "</link>");
    }

	private  void printHeader(PrintStream out) {
		out.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\"><html><head><title>blog</title></head><body bgcolor=\"black\" background=\"webpics/stars.gif\" alink=\"yellow\" link=\"yellow\" vlink=\"yellow\">");
		out.println("<TABLE border=0 cellPadding=0 cellSpacing=0 width=\"100%\"><TBODY><TR><TD vAlign=top width=\"150\"></td><td>");
	}

	private  void printItem(PrintStream out, BlogItem item) {
		out.println(getAsHTML(item));
	}

	private  void printFooter(PrintStream out) {
		out.println("</td><TD vAlign=top width=\"150\"></td></tr></tbody></table>");
		out.println("</body></html>");
	}


	public BlogItem fetchBlogItemFromUser() {
		BlogItem testItem = new BlogItem(1,new Date(),"jdshkfjhs");
		return testItem;
	}

	public static void main(String[] args) {
		Blogger blogger = new Blogger();
		blogger.doIt(new BlogDestination(), blogger);
	}

	public void doIt(BlogDestination destination, BlogCallback callback) {
        this.destination = destination;
		BlogItem storableItem = callback.fetchBlogItemFromUser();
		if (storableItem != null) {
			store(storableItem);
		}
        createBlog(destination);
    }

    public void createBlog(BlogDestination destination) {
        this.destination = destination;
		try {
            System.out.println(destination.getProject());
			PrintStream out = new PrintStream(new FileOutputStream(destination.getProject() + "-blog.html"));
			printHeader(out);
			List items = readItems();
			Iterator itemsItr = items.iterator();
			while (itemsItr.hasNext()) {
				BlogItem item = (BlogItem)itemsItr.next();
				printItem(out,item);
			}
			printFooter(out);
			con.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

    private String getAsXML(BlogItem item) {
        StringBuffer sb = new StringBuffer();
        sb.append("<item><description>");
        sb.append(item.getDescription());
        sb.append("</description></item>");
        return sb.toString();
    }

	private String getAsHTML(BlogItem item) {
		StringBuffer sb = new StringBuffer();
		if (!(previousDate.equals(item.getDate()))) {
			if (!firstItem) {
				sb.append("<P> <P><BR>");
			}
			firstItem = false;
			sb.append("<TABLE border=0 cellPadding=0 cellSpacing=0 width=\"100%\"><TBODY><TR><TD vAlign=center width=1><A href=\"" + destination.getWebserver());
			sb.append(item.getDateLink());
			sb.append(".html\"><IMG border=0 height=15 src=\"permalink.gif\" width=12></A> </TD> <TD vAlign=center width=\"100%\">&nbsp;<B><font color=\"white\"> ");
			sb.append(item.getDate());
			sb.append("</font></B></TD></TR> <TR> <TD colSpan=2> <HR SIZE=1> </TD></TR></TBODY></TABLE>");
			previousDate = item.getDate();
		}
		sb.append("<A name=a");
		sb.append(item.getKey());
		sb.append("></A> <TABLE cellPadding=1 width=\"100%\"> <TBODY> <TR> <TD vAlign=top><BR> <font color=\"white\">");
		sb.append(item.getDescription());
		sb.append("</font><BR> <FONT class=small color=gray size=-1>");
		sb.append(item.getTime());
		sb.append("&nbsp;&nbsp;<A href=\"" + destination.getWebserver() + "/");
		sb.append(item.getDateLink());
		sb.append(".html#a");
		sb.append(item.getKey());
		sb.append("\"><IMG border=0 height=9 src=\"woodsItemLink.gif\" width=7></A>&nbsp;&nbsp;</FONT> </TD> <TD align=right noWrap vAlign=top></TD></TR></TBODY></TABLE>");
		return sb.toString();
	}

	public void upload(BlogDestination destination) {
		this.destination = destination;
		String server   = destination.getServer();
		String username = destination.getUsername();
		String password = destination.getPassword();
		String remote   = destination.getRemote();
		String local    = destination.getLocal();

		FTPClient ftp = new FTPClient();
		ftp.addProtocolCommandListener(new PrintCommandListener(new PrintWriter(System.out)));

		try {
			int reply;
			ftp.connect(server);
			System.out.println("Connected to " + server + ".");
			reply = ftp.getReplyCode();
			if(!FTPReply.isPositiveCompletion(reply)) {
				ftp.disconnect();
				System.err.println("FTP server refused connection.");
				System.exit(1);
			}
		} catch(IOException e) {
			if(ftp.isConnected()) {
				try {
					ftp.disconnect();
				} catch(IOException f) {
					// do nothing
				}
			}
			System.err.println("Could not connect to server.");
			e.printStackTrace();
			System.exit(1);
		}

		__main:
		try {
			if(!ftp.login(username, password)) {
				ftp.logout();
				break __main;
			}
			System.out.println("Remote system is " + ftp.getSystemName());

			InputStream input;
			input = new FileInputStream(local);
			ftp.storeFile(remote, input);

			ftp.logout();
		} catch(FTPConnectionClosedException e) {
			System.err.println("Server closed connection.");
			e.printStackTrace();
		} catch(IOException e) {
			e.printStackTrace();
		} finally {

			if(ftp.isConnected()) {
				try {
					ftp.disconnect();
				} catch(IOException f) {
					// do nothing
				}
			}
		}
	}
}